@@ -4,7 +4,7 @@
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-# http://www.apache.org/licenses/LICENSE-2.0
+# https://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 from __future__ import absolute_import, unicode_literals, print_function
+import collections
 import hashlib
 import io
 import os
@@ -30,6 +31,7 @@ except ImportError:
 
 from twine.wheel import Wheel
 from twine.wininst import WinInst
+from twine import exceptions
 
 DIST_TYPES = {
     "bdist_wheel": Wheel,
@@ -61,24 +63,13 @@ class PackageFile(object):
         self.signed_basefilename = self.basefilename + '.asc'
         self.gpg_signature = None
 
-        blake2_256_hash = None
-        if blake2b is not None:
-            blake2_256_hash = blake2b(digest_size=256 // 8)
-        # NOTE(sigmavirus24): We may or may not be able to use blake2 so let's
-        # either use the methods or lambdas to do nothing.
-        blake_update = getattr(blake2_256_hash, 'update', lambda *args: None)
-        blake_hexdigest = getattr(blake2_256_hash, 'hexdigest', lambda: None)
-        md5_hash = hashlib.md5()
-        sha2_hash = hashlib.sha256()
-        with open(filename, "rb") as fp:
-            for content in iter(lambda: fp.read(io.DEFAULT_BUFFER_SIZE), b''):
-                md5_hash.update(content)
-                sha2_hash.update(content)
-                blake_update(content)
+        hasher = HashManager(filename)
+        hasher.hash()
+        hexdigest = hasher.hexdigest()
 
-        self.md5_digest = md5_hash.hexdigest()
-        self.sha2_digest = sha2_hash.hexdigest()
-        self.blake2_256_digest = blake_hexdigest()
+        self.md5_digest = hexdigest.md5
+        self.sha2_digest = hexdigest.sha2
+        self.blake2_256_digest = hexdigest.blake2
 
     @classmethod
     def from_filename(cls, filename, comment):
@@ -88,7 +79,7 @@ class PackageFile(object):
                 meta = DIST_TYPES[dtype](filename)
                 break
         else:
-            raise ValueError(
+            raise exceptions.InvalidDistribution(
                 "Unknown distribution format: '%s'" %
                 os.path.basename(filename)
             )
@@ -161,13 +152,15 @@ class PackageFile(object):
 
     def add_gpg_signature(self, signature_filepath, signature_filename):
         if self.gpg_signature is not None:
-            raise ValueError('GPG Signature can only be added once')
+            raise exceptions.InvalidDistribution(
+                'GPG Signature can only be added once'
+            )
 
         with open(signature_filepath, "rb") as gpg:
             self.gpg_signature = (signature_filename, gpg.read())
 
     def sign(self, sign_with, identity):
-        print("Signing {0}".format(self.basefilename))
+        print("Signing {}".format(self.basefilename))
         gpg_args = (sign_with, "--detach-sign")
         if identity:
             gpg_args += ("--local-user", identity)
@@ -175,3 +168,70 @@ class PackageFile(object):
         subprocess.check_call(gpg_args)
 
         self.add_gpg_signature(self.signed_filename, self.signed_basefilename)
+
+
+Hexdigest = collections.namedtuple('Hexdigest', ['md5', 'sha2', 'blake2'])
+
+
+class HashManager(object):
+    """Manage our hashing objects for simplicity.
+
+    This will also allow us to better test this logic.
+    """
+
+    def __init__(self, filename):
+        """Initialize our manager and hasher objects."""
+        self.filename = filename
+        try:
+            self._md5_hasher = hashlib.md5()
+        except ValueError:
+            # FIPs mode disables MD5
+            self._md5_hasher = None
+
+        self._sha2_hasher = hashlib.sha256()
+        self._blake_hasher = None
+        if blake2b is not None:
+            self._blake_hasher = blake2b(digest_size=256 // 8)
+
+    def _md5_update(self, content):
+        if self._md5_hasher is not None:
+            self._md5_hasher.update(content)
+
+    def _md5_hexdigest(self):
+        if self._md5_hasher is not None:
+            return self._md5_hasher.hexdigest()
+        return None
+
+    def _sha2_update(self, content):
+        if self._sha2_hasher is not None:
+            self._sha2_hasher.update(content)
+
+    def _sha2_hexdigest(self):
+        if self._sha2_hasher is not None:
+            return self._sha2_hasher.hexdigest()
+        return None
+
+    def _blake_update(self, content):
+        if self._blake_hasher is not None:
+            self._blake_hasher.update(content)
+
+    def _blake_hexdigest(self):
+        if self._blake_hasher is not None:
+            return self._blake_hasher.hexdigest()
+        return None
+
+    def hash(self):
+        """Hash the file contents."""
+        with open(self.filename, "rb") as fp:
+            for content in iter(lambda: fp.read(io.DEFAULT_BUFFER_SIZE), b''):
+                self._md5_update(content)
+                self._sha2_update(content)
+                self._blake_update(content)
+
+    def hexdigest(self):
+        """Return the hexdigest for the file."""
+        return Hexdigest(
+            self._md5_hexdigest(),
+            self._sha2_hexdigest(),
+            self._blake_hexdigest(),
+        )
