@@ -4,7 +4,7 @@
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-# http://www.apache.org/licenses/LICENSE-2.0
+# https://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
@@ -28,7 +28,7 @@ from requests_toolbelt.utils import user_agent
 
 import twine
 
-KEYWORDS_TO_NOT_FLATTEN = set(["gpg_signature", "content"])
+KEYWORDS_TO_NOT_FLATTEN = {"gpg_signature", "content"}
 
 LEGACY_PYPI = 'https://pypi.python.org/'
 LEGACY_TEST_PYPI = 'https://testpypi.python.org/'
@@ -47,7 +47,8 @@ class ProgressBar(tqdm):
 
 
 class Repository(object):
-    def __init__(self, repository_url, username, password):
+    def __init__(self, repository_url, username, password,
+                 disable_progress_bar=False):
         self.url = repository_url
         self.session = requests.session()
         self.session.auth = (username, password)
@@ -55,6 +56,7 @@ class Repository(object):
         for scheme in ('http://', 'https://'):
             self.session.mount(scheme, self._make_adapter_with_retries())
         self._releases_json_data = {}
+        self.disable_progress_bar = disable_progress_bar
 
     @staticmethod
     def _make_adapter_with_retries():
@@ -106,7 +108,7 @@ class Repository(object):
             "protocol_version": "1",
         })
 
-        print("Registering {0}".format(package.basefilename))
+        print("Registering {}".format(package.basefilename))
 
         data_to_send = self._convert_data_to_list_of_tuples(data)
         encoder = MultipartEncoder(data_to_send)
@@ -130,7 +132,7 @@ class Repository(object):
 
         data_to_send = self._convert_data_to_list_of_tuples(data)
 
-        print("Uploading {0}".format(package.basefilename))
+        print("Uploading {}".format(package.basefilename))
 
         with open(package.filename, "rb") as fp:
             data_to_send.append((
@@ -140,7 +142,8 @@ class Repository(object):
             encoder = MultipartEncoder(data_to_send)
             with ProgressBar(total=encoder.len,
                              unit='B', unit_scale=True, unit_divisor=1024,
-                             miniters=1, file=sys.stdout) as bar:
+                             miniters=1, file=sys.stdout,
+                             disable=self.disable_progress_bar) as bar:
                 monitor = MultipartEncoderMonitor(
                     encoder, lambda monitor: bar.update_to(monitor.bytes_read)
                 )
